import { Record, List } from 'immutable';
import { LOCATION_CHANGED } from '../util/router';
import createReducer from '../util/createReducer';
import * as actions from '../actions';

const TabRecord = Record({
  server: null,
  name: null
});

class Tab extends TabRecord {
  isChannel() {
    return this.name && this.name.charAt(0) === '#';
  }
}

const State = Record({
  selected: new Tab(),
  history: List()
});

function selectTab(state, action) {
  const tab = new Tab(action);
  return state
    .set('selected', tab)
    .update('history', history => history.push(tab));
}

export const getSelectedTab = state => state.tab.selected;

export default createReducer(new State(), {
  [actions.SELECT_TAB]: selectTab,

  [actions.PART](state, action) {
    return state.set('history', state.history.filter(tab =>
      !(tab.server === action.server && tab.name === action.channels[0])
    ));
  },

  [actions.CLOSE_PRIVATE_CHAT](state, action) {
    return state.set('history', state.history.filter(tab =>
      !(tab.server === action.server && tab.name === action.nick)
    ));
  },

  [actions.DISCONNECT](state, action) {
    return state.set('history', state.history.filter(tab => tab.server !== action.server));
  },

  [LOCATION_CHANGED](state, action) {
    const { route, params } = action;
    if (route === 'chat') {
      return selectTab(state, params);
    }

    return state.set('selected', new Tab());
  }
});
